/**********************************************************************
 *
 * This file is part of HBCI4Java.
 * Copyright (c) 2019 Olaf Willuhn
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 **********************************************************************/

package org.kapott.hbci.dialog;

/**
 * Diese Klasse enthaelt die verschiedenen Event-Arten.
 */
public enum DialogEvent
{
    /**
     * Wird gesendet, bevor HBCI4Java die Nachricht erzeugt aber noch nicht an die Bank gesendet hat.
     * Kann mehrfach auftreten, wenn z.Bsp. die Dialog-Initialisierung nach einer Neuwahl des TAN-Verfahrens (aus Code 3920) wiederholt werden muss.
     */
    MSG_CREATE,
    
    /**
     * Wird versendet, bevor HBCI4Java die Nachricht an die Bank gesendet hat und das Response vorliegt.
     * Kann mehrfach auftreten, wenn z.Bsp. die Dialog-Initialisierung nach einer Neuwahl des TAN-Verfahrens (aus Code 3920) wiederholt werden muss.
     */
    MSG_CREATED,
    
    /**
     * Wird versendet, nachdem HBCI4Java die Nachricht an die Bank gesendet hat und das Response vorliegt.
     * Kann mehrfach auftreten, wenn z.Bsp. die Dialog-Initialisierung nach einer Neuwahl des TAN-Verfahrens (aus Code 3920) wiederholt werden muss.
     */
    MSG_SENT,
    
    /**
     * Wird gesendet, nachdem die Dialog-Initialisierung abgeschlossen ist und bevor die eigentlichen Geschaeftsvorfaelle an die Bank gesendet werden.
     */
    JOBS_CREATED,
}


